-- Most of the comments to the code (and the "well written" part of it :-) are by Phanx, whom I thanks for the helps and patience in explaining. 

local AggromonCFG = AggromonCFG or {}

local cmd, out, args -- cmd args
local prgname = "|cffffd200aggromon|r"
local string_format = string.format
local aggromon_status, aggromon_status_old, aggromon_value, aggromon_defvol, aggromon_snd, aggromon_summary

-- Frame related vars
local bgfile = "Interface\\ChatFrame\\ChatFrameBackground"
local edgefile = "Interface\\Tooltips\\UI-Tooltip-Border"
local framefontn, framefonth, framefontf = GameFontNormal:GetFont()

-- Define all of the locals (functions) first, and then set their values in whatever order
local Aggromon_print_summary, Aggromon_check
local frame_cleu, frame_cleu2, frame_al
local Aggromon_Show, Aggromon_Value
local aggromon_lvl, aggromon_color, aggromon_soundfile

local tostring = tostring

aggromon_lvl = {
	"in aggro, but not tanking anything",
	"not tanking anything, but have higher threat than tank on at least one unit",
	"insecurely tanking at least one unit, but not securely tanking anything",
	"securely tanking at least one unit",
}

	-- nil = unit is not on any other unit's threat table.
	-- 0 = not tanking anything.
	-- 1 = not tanking anything, but have higher threat than tank on at least one unit.
	-- 2 = insecurely tanking at least one unit, but not securely tanking anything.
	-- 3 = securely tanking at least one unit. 
	-- descriptions taken from: http://www.wowwiki.com/API_UnitThreatSituation

	-- nil  
	-- 0 = Unit has less than 100% raw threat (default UI shows no indicator)
    -- 1 = Unit has 100% or higher raw threat but isn't mobUnit's primary target (default UI shows yellow indicator)
    -- 2 = Unit is mobUnit's primary target, and another unit has 100% or higher raw threat (default UI shows orange indicator)
    -- 3 = Unit is mobUnit's primary target, and no other unit has 100% or higher raw threat (default UI shows red indicator)
	-- descriptions taken from: http://wowprogramming.com/docs/api/UnitThreatSituation
	
aggromon_color = {
	{1,1,0},
	{1,0.5,0},
	{1,0.2,0},
	{1,0,0},
}

	
aggromon_soundfile = {
	"Interface\\AddOns\\Aggromon\\sounds\\babe.ogg",
	"Interface\\AddOns\\Aggromon\\sounds\\zenham.ogg",
	"Sound\\Doodad\\BellTollAlliance.wav",
	"Sound\\interface\\AuctionWindowClose.wav",
	
	-- Sounds taken from banzai-alert: http://www.wowace.com/addons/banzai-alert/ 
}

function Aggromon_check()
		
	if AggromonCFG["enabled"] == false then
		frame_cleu:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		frame_cleu2:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	else
		frame_cleu:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
	return		

end
	
function Aggromon_print_summary ()

		if AggromonCFG["show"] == false then	
			aggromon_summary = "[show no] "
		else
			aggromon_summary = "[show yes] "
		end
		
		if AggromonCFG["print"] == false then	
			aggromon_summary = aggromon_summary .. "[print no] "
		else
			aggromon_summary = aggromon_summary .. "[print yes] "
		end
		
		if AggromonCFG["sound"] == false then	
			aggromon_summary = aggromon_summary .. "[sound no] "
		else
			aggromon_summary = aggromon_summary .. "[sound yes] "
		end

		if AggromonCFG["enabled"] == false then			
			print(string_format("%s: |cffff0000disabled|r. Type /%s enable", prgname, prgname))
		else
			print(string_format("%s: |cff228B22enabled|r %s" , prgname, aggromon_summary))
		end 

end
		
		
-- Frame colored based on aggro
Aggromon_Show = CreateFrame("Frame","Aggromon_Show",UIParent)
	Aggromon_Show:SetBackdrop({bgFile = bgfile, edgeFile = edgefile, edgeSize = 18, insets = {left = 5, right = 5, top = 5, bottom = 5}})
	Aggromon_Show:SetBackdropColor(0,0,0,0.8)
	Aggromon_Show:SetWidth(40)
	Aggromon_Show:SetHeight(40)
	Aggromon_Show:SetPoint("CENTER",UIParent)
	Aggromon_Show:EnableMouse(true)
	Aggromon_Show:SetMovable(true)
	Aggromon_Show:RegisterForDrag("LeftButton")
	Aggromon_Show:SetScript("OnDragStart", function(self) self:StartMoving() end)
	Aggromon_Show:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)		
	Aggromon_Show:SetClampedToScreen(true)

	
Aggromon_Value = Aggromon_Show:CreateFontString("Aggromon_Show")
	Aggromon_Value:SetPoint("CENTER", Aggromon_Show,"CENTER",0, 0)
	Aggromon_Value:SetFont(GameFontNormal:GetFont(),(framefonth))
	
	
-- Frame to register the global vars at the ADDON_LOADED event.	
frame_al = CreateFrame("FRAME", "aggromon_al")
frame_al:RegisterEvent("ADDON_LOADED")	
frame_al:SetScript("OnEvent", function(self, event, arg1, ...)
	
	if event == "ADDON_LOADED" and arg1 == "Aggromon" then	
		
		AggromonCFG["enabled"] = AggromonCFG["enabled"] or true
		AggromonCFG["sound"] = AggromonCFG["sound"] or true
		AggromonCFG["print"] = AggromonCFG["print"] or false
		AggromonCFG["show"] = AggromonCFG["show"] or true
		AggromonCFG["soundfile"] = AggromonCFG["soundfile"] or aggromon_soundfile[1]
		
		if AggromonCFG["show"] == true then 
			Aggromon_Show:Show()
		else
			Aggromon_Show:Hide()
		end
		
		Aggromon_print_summary()
		
		self:UnregisterEvent("ADDON_LOADED")		

	end	
end)

-- Frame to parse UNIT_THREAT_SITUATION_UPDATE	
frame_cleu = CreateFrame("FRAME", "aggroframe_cleu")
frame_cleu:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu:RegisterEvent("PLAYER_ENTERING_WORLD")
frame_cleu:SetScript("OnEvent", function(self, event, ...)


	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then
		Aggromon_check()
	end 
	
	aggromon_status = UnitThreatSituation("player")
	
	-- show the colored frame
	if AggromonCFG["show"] == true then	
		if aggromon_status then
			Aggromon_Show:SetBackdropColor(aggromon_color[aggromon_status+1][1],aggromon_color[aggromon_status+1][2],aggromon_color[aggromon_status+1][3],0.8)
		else
			Aggromon_Show:SetBackdropColor(0,1,0,0.8)			
		end	
		
		
	end 
	
	-- print on the console
	if AggromonCFG["print"] == true then 
		if aggromon_status then 
			DEFAULT_CHAT_FRAME:AddMessage(string_format("aggromon: [threat lvl:%i] [target:%s] %s", aggromon_status, aggromon_value, aggromon_lvl[aggromon_status+1]), aggromon_color[aggromon_status+1][1],aggromon_color[aggromon_status+1][2],aggromon_color[aggromon_status+1][3]) 
		else  
			DEFAULT_CHAT_FRAME:AddMessage("aggromon: [threat nil] no aggro, unit is not on any other unit's threat table.",0,1,0) 
		end
	end 

	-- play the old "Aggro!" ... It miss me so much ...  ! :-)
	if AggromonCFG["sound"] == true then 
		if aggromon_status and aggromon_status >= 1 and aggromon_status_old == nil then
			PlaySoundFile(aggromon_soundfile[1], "Master") 	
		end
	end 	
	
	-- store the actual aggro level
	aggromon_status_old = aggromon_status
	
end)
	

-- create a second frame to register also the COMBAT_LOG to update the value aggromon_value	
frame_cleu2 = CreateFrame("FRAME", "aggroframe_cleu2")
frame_cleu2:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu2:RegisterEvent("PLAYER_ENTERING_WORLD")
frame_cleu2:SetScript("OnEvent", function(self, event, ...)

	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then
		Aggromon_check()
	end 

	_,_,aggromon_value,_,_ = UnitDetailedThreatSituation("player", "target");
	aggromon_value = tostring(string_format("%.0f",(aggromon_value or 0)))
	Aggromon_Value:SetText(aggromon_value)

end)
	
SLASH_AGGROMON1 = "/aggromon";
SlashCmdList["AGGROMON"] = function(args) 
	cmd, arg1 = string.split(" ", args:lower(), 2)
	
	if cmd == "enable" then
		AggromonCFG["enabled"] = true
		AggromonCFG["show"] = true
		Aggromon_Show:Show()
		Aggromon_check()
	
	elseif cmd == "disable" then
		AggromonCFG["enabled"] = false
		AggromonCFG["show"] = false
		Aggromon_Show:Hide()
		Aggromon_check()

	elseif cmd == "show" then
		if arg1 == "yes" then 
			AggromonCFG["show"] = true
			Aggromon_Show:Show()
		else
			AggromonCFG["show"] = false
			Aggromon_Show:Hide()
		end
	
	elseif cmd == "print" then
		if arg1 == "yes" then 
			AggromonCFG["print"] = true
		else
			AggromonCFG["print"] = false
		end

	elseif cmd == "sound" then
		if arg1 == "yes" then 
			AggromonCFG["sound"] = true
		else
			AggromonCFG["sound"] = false
		end

	elseif cmd == "legenda" then
		local index
		print(prgname .. " colors:")
		DEFAULT_CHAT_FRAME:AddMessage("aggromon: [threat nil] no aggro, unit is not on any other unit's threat table.",0,1,0) 
		for index = 0, 3, 1 do
			DEFAULT_CHAT_FRAME:AddMessage(string_format("aggromon: [threat lvl:%i] %s", index, aggromon_lvl[index+1]), aggromon_color[index+1][1],aggromon_color[index+1][2],aggromon_color[index+1][3]) 
		end
	
	else
	
		print(prgname .. " commands")
		print("/aggromon - This help")
		print("/aggromon enable - Enable aggro alert")
		print("/aggromon disable - Disable aggro alert")
		print("/aggromon legenda - Show colors legenda")
		print("/aggromon show [yes|no] - Show a colored frame aggro based.")
		print("/aggromon print [yes|no] - Print aggro variation on default chat")
		print("/aggromon sound [yes|no] - Play Aggro! on gaining aggro")
	
	end
	
	Aggromon_print_summary()
	
end